# -*- coding: utf-8 -*-
from unittest.mock import patch
from uuid import uuid4
from werkzeug.urls import url_encode

from odoo.addons.saas_worker import metabase
from odoo.addons.saas_worker.const import SAAS_BASE_URL, SCHEME, TIMEOUT
from odoo.addons.web.tests.test_login import TestWebLoginCommon
from odoo.tests.common import new_test_user
from ..controllers.signup import SignupController


class TestUserInvitations(TestWebLoginCommon):
    def setUp(self):
        super().setUp()
        self.Controller = SignupController()
        self.startPatcher(patch('odoo.addons.saas_worker.metabase.get_by_host',
                                side_effect=lambda host: metabase.get(self.cr.dbname)))

    def test_confirm_not_authenticated(self):
        url = '/openerp_enterprise/confirm_user?db=none&signup_email=user@doma.in&token=t0k3n'
        resp = self.url_open(url, allow_redirects=False)
        redirect_login_url = f'/web/login?{url_encode({"login": "user@doma.in", "redirect": url})}'
        redirect_url = f'{self.base_url()}/web/session/logout?{url_encode({"redirect": redirect_login_url})}'
        self.assertEqual(resp.status_code, 303)
        self.assertEqual(resp.headers['location'], redirect_url)

    @patch('requests.post')
    @patch('odoo.addons.openerp_enterprise.controllers.signup.SaasAuth')
    def test_confirm_authenticated_as_other_user(self, MockSaasAuth, mock_requests_post):
        db_uuid = uuid4()
        user = new_test_user(self.env, login='user@doma.in', password='user@doma.in')
        self.authenticate('user@doma.in', 'user@doma.in')
        mock_requests_post.return_value.json.return_value = {"result": {'db_uuid': db_uuid}}

        querystring = url_encode({
            'db': 'testdb',
            'signup_email': user.email,
            'token': 't0k3n',
        })
        url = f'/openerp_enterprise/confirm_user?{querystring}'
        resp = self.url_open(url, allow_redirects=False)

        mock_requests_post.assert_called_with(f'{SCHEME}://testdb.{SAAS_BASE_URL}/saas_trial/set_oauth_uid',
                                              json={'jsonrpc': '2.0', 'params': {
                                                  'login': 'user@doma.in',
                                                  'token': 't0k3n',
                                                  'oauth_uid': user.id,
                                              }},
                                              auth=MockSaasAuth(),
                                              timeout=TIMEOUT)

        redirect_url = f'{self.base_url()}/my/databases/connect/{db_uuid}'
        self.assertEqual(resp.status_code, 303)
        self.assertEqual(resp.headers['location'], redirect_url)

    @patch('requests.post')
    def test_confirm_authenticated(self, mock_requests_post):
        user = new_test_user(self.env, login='user@doma.in', password='user@doma.in')
        self.authenticate('admin', 'admin')

        querystring = url_encode({
            'db': 'testdb',
            'signup_email': user.email,
            'token': 't0k3n',
        })
        url = f'/openerp_enterprise/confirm_user?{querystring}'
        resp = self.url_open(url, allow_redirects=False)

        mock_requests_post.assert_not_called()

        login_url = f'/web/login?{url_encode({"login": "user@doma.in", "redirect": url})}'
        redirect_url = f'{self.base_url()}/web/session/logout?{url_encode({"redirect": login_url})}'
        self.assertEqual(resp.status_code, 303)
        self.assertEqual(resp.headers['location'], redirect_url)
